<?php


	class FPDB {
		
		var $_indexer = null;
		var $_query = array();
		
	
		function FPDB() {
			// constructor
		}
		
		function init() {
			if (!$this->_indexer) {
				$this->_indexer =& new entry_indexer();
				$this->_categories = entry_categories_get();
			}
		}
		
		function reset($queryId=null) {
			
			switch ($queryId) {
				case null: 	$this->_query = array(); break;
				default:	unset($this->_query[$queryId]);
			}
			
		
		}
		
		/**
		 * function query
		 * @param mixed params
		 * $params may be an associative array or a query string with the following syntax:
		 * 'key:val,key:val,key:val';
		 * example: <code>$params = 'start:0,count:5';<br />
		 *		is a convenient way to express $params = array('start'=>0,'count'=>5);</code>
		 *
		 * Valid parameters:
		 * 
		 * start	(int) first entry to show (counting from 0 to the total number of entries).
		 *		Defaults to 0.
		 *
		 * count	(int) displacement from start (e.g. for the first 5 entries, you'll have start=0 and count=5).
		 *		Defaults to $blog_config['MAXENTRIES'] 
		 *
		 * page		(int) page number (counting from 1 to n=ceil(num_entries/maxentries_setting))
		 *		This is a shortcut for setting both start and count and overrides them, if they're set too
		 * 
		 * id		(string) entry or static page id
		 *
		 * get		(string) 'entry' or 'static' defaults to 'entry'.
		 *
		 * y		(string) two digit year (06 means 2006)
		 * m		(string) two digit month (06 means June); 'y' must be set
		 * d		(string) two digit for day; 'y' and 'm' must be set
		 *
		 *
		 * fullparse	(bool) non-full-parsed entries get their values directly from the indexed list (or <em>cache</em>).
		 *		These values are 'subject', 'content' and 'categories'. If you need any of the other values, you'll need to
		 *		set fullparse=true; defaults to true.
		 *
		 */			 
		
		function query($params=array()) {
			
			static $queryId=-1;
			$queryId++;
			 			 
			 if (is_string($params))
				$params = utils_kexplode(
				strtolower($params), ',:', false);
			
			$this->_query[$queryId]['params'] = $params;
			$this->_query[$queryId]['counter']=0;
			
			return $queryId;
			
		
		}
		
		function doquery($queryId=0) {
			
			if (!$this->_query[$queryId]) return false;
			$this->_query[$queryId]['counter']++;
			
			
			$this->init();

			global $blog_config;
			
			$page = null;
			$cat = null;
			$start = 0;
			$count	= $blog_config['MAXENTRIES'];
			$y = $m = $d = $entry = $static = null;
			
			if (isset($this->_query[$queryId]['params']['fullparse'])
				&& is_string($fullparse =& $this->_query[$queryId]['params']['fullparse'])) {
				$fullparse = ($fullparse != 'false');	
			}
			//$fullparse = true;
			
			$id = null;
			
			/// IMPORT PARAMETERS INTO SCOPE ///
			extract($this->_query[$queryId]['params']);
			
			/// ---------------------------- ///
			
			//$this->_query[$this->_queryId]['list']='entry';
			$isEntry = true;
			$isStatic = false;
			
			if ($id) {
				
				if (utils_validateinput($id)) {
					
					if (entry_exists($id)) {
						$isEntry = true;
					} else {
						// unset($this->_query[$queryId]);
						
						// FIXME: temporary fix
						utils_redirect();
						
						return false;
					}
					
				
				}
				
			}
			
			$link = '';
				
			
			
			if ($isEntry) {
				// ---> preparing data
			
				$obj =& $this->_indexer;
				$entry_arr = $obj->getList();
				
				//$this->_queryId=$queryId;
				
				$total = count($entry_arr);
			
				//krsort($entry_arr); 
				
				
				
				// ---> clean up parameters
				
				$filterid='';
				
				// ---> permalink trigger
				
				if ($id) {
				
					$this->_query[$queryId]['curId'] = $id;	
					$cachedlist = $entry_arr;
					$entry_arr = array_keys($entry_arr);
					
					$start = array_search($id, $entry_arr);
					
					
					list($ok,$next) = $this->_getNext($entry_arr, $cachedlist, $start);
					
					
					if ($ok>0) {
						//$next = $entry_arr[$start+$i];
						$this->_query[$queryId]['nextLink'] = '?entry=' . $next;
						$this->_query[$queryId]['nextLinkCaption'] = $cachedlist[$next]['subject'] . ' &raquo;';
					}
					
					
					// find prev available
					
					list($ok,$prev) = $this->_getPrev($entry_arr, $cachedlist, $start);
					
					if ($ok>0) {
						//$prev = $entry_arr[$start-1];
						$this->_query[$queryId]['prevLink'] = '?entry=' . $prev;
						$this->_query[$queryId]['prevLinkCaption'] = '&laquo; ' . $cachedlist[$prev]['subject'];
					}
					
					$start = 0;
					$count = 1;
					
					$entry_arr = array($id);
					
				} else { // begin searching
					
					// setup count
					
					if ($count<0) {
						$count=$total;
					}
					if (isset($page)) {
						if ($page<1) {
							$page=1;
						}
						$start = ($page-1) * $blog_config['MAXENTRIES'];
					}
					
						
						
					// ---> filter by date: setup glob
					
					$filteredkeys = array();
					$filterbydate = false;
					
					if (isset($y) && strlen($y)<=2) {
						$y=str_pad($y,2,'0', STR_PAD_LEFT);
						$link .= "&amp;y=$y";
						
						if (!isset($m) || (isset($m) && strlen($m)>2)) { 
							$m=''; 
						} else { 
							$m=str_pad($m,2,'0', STR_PAD_LEFT);
							$link .= "&amp;m=$m";
							
							$filterbydate = true;
							
							$obj =& new fs_filelister(CONTENT_DIR . "$y/$m"); 
							$filteredkeys = $obj->getList();
							
							if (!isset($d) || (isset($d) && strlen($d)>2)) { 
								$d='';
							} else {
								$d=str_pad($d,2,'0', STR_PAD_LEFT);
								$link .= "&amp;d=$d";
							}
						}
						
						$filterid = "entry$y$m$d*"; //old?
								
					}
					
					
					
					// ---> filter by category
					
					// cat is set but is non-valid
					$flags = entry_flags_get();
					$newarr = array();
					
					if (!isset($cat)) $cat = 0;
					else $link .= "&amp;category=$cat";
					
					if (!is_numeric($cat) && $cat!== 'all' && !in_array($cat, $flags))
						return false;
					else {
						if ($cat != 0 && !isset($this->_categories['rels'][$cat])) {
							return false;
						}
					}
					
					
					if ($filterbydate) {
						$newarr = array();
						
						rsort($filteredkeys);
						
						foreach($filteredkeys as $key) {
							$key = substr($key, 0, -4);
							$newarr[$key] = $entry_arr[$key]; 
						}
						$entry_arr = $newarr;
					}
					
					
					$newarr = array();
					
					// quick and dirty... todo: optimize, it doesn't keep track of drafts
					$limit = $start + $count + 1;
					
					reset($entry_arr);
					$i = 0;
					
					while((list($id, $entry) = each($entry_arr))  
							&& $i < $limit) {
						
						$keep = true;
						
						// applying date filter (if any)
						
						
						if ($filterid) {
							$keep = fnmatch($filterid, $id);
						} 
						
						
						// filtering hidden categories (flags) and eventually normal categories
						
						if ($keep) {
							$categs = $entry['categories'];
							// negative categories are hidden by default
							// category 0 means "all" (excepted the hidden ones)
							// negative categories can be accessed only by specifying
							// their correct id
							// using 'all' will retrieve all the entries.
							
								
							//$keep = !(array_intersect(array('draft'), $categs));
							if (is_array($categs))
								$keep = !in_array('draft', $categs);
							else trigger_error("$id : ". gettype($categs));
							
							
							// thanks to cimangi for noticing this
							if ($cat>0 && $keep) {
								
								$relations =& $this->_categories['rels'];
								
								$keep = array_intersect($relations[$cat], $categs) ;
							}
							
							
							if (user_loggedin()) { // only a loggedin user can see special categories
								
								if ($cat === 'all')
									$keep = true;
								
								elseif (!is_numeric($cat)) {
									$keep = fnmatch("*$cat*", implode(',',$categs));	
								}
									
							}
							
							
							if (isset($callback)) {
								$keep = call_user_func($callback, array($id, $entry));
							}
							
							if ($keep) {
								$i++;
								$newarr[$id] = $entry_arr[$id];
							}	
							
							
								
						}
						
						
						// this entry has been analyzed, so we can free
						// the memory it's using.
						unset($entry_arr[$id]);
						
					}
		
					$entry_arr = $newarr;
					unset($newarr);
					
					
					
					if ($filterbydate)
						{ krsort($entry_arr); } 
						
						
					
					if (!$entry_arr) {
						// no match
						
						$start = 0;
						$count = 0;
						
						return false;
					}
					
					
					$hasMore = true;
					
					if ($limit == ($tot = count($entry_arr))) {
						$tot--;
						array_pop($entry_arr);
						$hasMore = true;
					} elseif ($limit > ($tot = count($entry_arr))){// && (($cat!=='all') || isset($y) || isset($id))) {
						$total = $tot;
						$hasMore = false;
					}
					
					$cachedlist = $entry_arr;
					$entry_arr = array_keys($cachedlist);
		
					if (isset($page)) {
					
						if ($hasMore) {
							$this->_query[$queryId]['nextLink'] = '?paged=' . ($page+1) . $link;
							$this->_query[$queryId]['nextLinkCaption'] = $GLOBALS['lang']['main']['nextpage'];
						}
						if ($page > 1) {
							$this->_query[$queryId]['prevLink'] = '?paged=' . ($page-1) . $link;
							$this->_query[$queryId]['prevLinkCaption'] = $GLOBALS['lang']['main']['prevpage'];
						}
					
					} else {							
						
					
						if ($hasMore) {
						
							$this->_query[$queryId]['nextLink'] = '?start=' . ($start+$count) . $link;
							$this->_query[$queryId]['nextLinkCaption'] = $GLOBALS['lang']['main']['nextpage'];
						}
						if ($start-$count >= 0) {
							$this->_query[$queryId]['prevLink'] = '?start=' . ($start-$count) . $link;
							$this->_query[$queryId]['prevLinkCaption'] = $GLOBALS['lang']['main']['prevpage'];
						}
				
							
							
					}
					
					if (($v = $total-($start+$count)) < 0)
						$count += $v;
				}
				
				
			} else {
				// no match
				$start = 0;
				$count = 0;
				$entry_arr = array();
			}
			
				
			//$this->_hasmore = ($this->_max < count($entry_arr));
			$this->_query[$queryId]['ptr'] = $start;
			$this->_query[$queryId]['max'] = $count+$start;
			$this->_query[$queryId]['entryarr'] = $entry_arr;
			$this->_indexedlist = $this->_indexer->getList();
			
			
			//print_r($entry_arr);
			
			return (bool)$entry_arr;
			

		}
		
		
		
		
		function _getNext($entry_arr, $cachedlist, $start) {
			
			$ok = 0;
			$next = null;
			// find next available
			
			$i=1;
			do {
				if (isset($entry_arr[$start+$i])) {
					$next = $entry_arr[$start+$i];
					$ok = !in_array('draft', $cachedlist[$next]['categories']);
					$i++;
				} else { $ok = -1; }
			} while ( !$ok );
			
			return array($ok, $next, $i);
			
		}
		
		function _getPrev($entry_arr, $cachedlist, $start) {
				
			$ok = 0;
			$prev = null;

			$i=1;
			do {
				if (isset($entry_arr[$start-$i])) {
					$prev = $entry_arr[$start-$i];
					$ok = !in_array('draft', $cachedlist[$prev]['categories']);
					$i--;
				} else { $ok = -1; }
			} while ( !$ok );
			
			return array($ok, $prev, $i);
			
		}
		
		// "get" functions. todo: move out?
		
		
		function &getEntry($queryId=0) {

			$cachedlist =& $this->_indexedlist;

			$entryid = $this->_query[$queryId]['entryarr'][ $this->_query[$queryId]['ptr'] ];
			$this->_query[$queryId]['ptr'] += 1;
			
			$this->_query[$queryId]['curId'] = $entryid;
		
			
			
			$fullparse = isset($this->_query[$queryId]['params']['fullparse'])
			  && $this->_query[$queryId]['params']['fullparse'];
			 
			if ($fullparse) {
				
				$cont = array();
				
				if ($this->_query[$queryId]['counter']>1) {
					$cont = $this->_query[$queryId]['localcache'][$entryid];
				} else {
					$cont = entry_parse($entryid);
					$this->_query[$queryId]['localcache'][$entryid] = $cont;
				}
					
				
				// increment views if is single entry (and if this is the first time we run the query)
				// is single entry if isset "id" parameter
	
				$incrViews = isset($this->_query[$queryId]['params']['id']) && $this->_query[$queryId]['counter']==1;
				
				entry_get_views($entryid, $cont, $incrViews);
				
				
				$this->_query[$queryId]['commentindexer'] =& entry_get_comments($entryid, $cont);
				$this->_query[$queryId]['hasComments'] = $cont['comments'];
				
			} else {
				
				$cont = $cachedlist[$entryid];
				
			}
					
			
			//$cont = $this->parse($entryid, 'entry', $this->_query[$queryId]['params']);
			
			$this->_query[$queryId]['hasComments'] = isset($cont['commentcount'])? $cont['commentcount'] : false;
			
			$var = array(&$entryid, &$cont);
			
			return $var;
				
		}
		
		function getCurrentId($queryId=0) {
			return $this->_query[$queryId]['curId'];
		}

		function entryHasComments($queryId=0) {
			return $this->_query[$queryId]['hasComments'];
		}
		
		function entryIsSingle($queryId=0) {
			isset($this->_query[$queryId]['params']['id']);
		}
		
		function hasMore($queryId=0) {
			$hasmore = isset($this->_query[$queryId]) && $this->_query[$queryId]['ptr'] < $this->_query[$queryId]['max'];
			return $hasmore;
			
		}

		function getNextLink($queryId=0) {
			 return apply_filters('next_link', array($this->_query[$queryId]['nextLinkCaption'],$this->_query[$queryId]['nextLink']));
		}

		function getPrevLink($queryId=0) {
			return apply_filters('prev_link', array($this->_query[$queryId]['prevLinkCaption'], $this->_query[$queryId]['prevLink']));
		}
		
		function getEntryComments($queryId=0) {
			return (bool)($this->_query[$queryId]['comments'] = $this->_query[$queryId]['commentindexer']->getList());
		}

		function hasMoreComments($queryId=0) {
			if (is_array($this->_query[$queryId]['comments'])) {
				return current($this->_query[$queryId]['comments']) !== false;
			}
			else 
				return false;
		}

		function &getComment($queryId=0) {
			list($k, $id) = each($this->_query[$queryId]['comments']);
			$comment = comment_parse($this->_query[$queryId]['curId'],$id); 
			$couplet = array(&$id, &$comment);
			
			return $couplet;
		}
		
	}

	
	// SMARTY FUNCTIONS ----------------------------------------------------
	
	
	function smarty_block_entries($params, $content, &$smarty, &$repeat) {
		global $fpdb;
		
		$show = false;
		
		$smarty->assign('prev_entry_day', '');
		
		if ($repeat) {
		
			if (isset($params['alwaysshow']) && $params['alwaysshow']) {
				//$fpdb->doquery();
				$repeat = false;
				return $content;
			}
			
			$show = $fpdb->doquery();
		
		} else {
			
			if (!isset($fpdb->_query[0]['comments']) || !$fpdb->_query[0]['comments']) 
				$fpdb->reset(0);
				$show = true;
			
		}
		
		
		
		
		if ($show)
			return $content;
		
	}
			
	
	function smarty_block_entry($params, $content, &$smarty, &$repeat) {
		global $fpdb;
		
		// clean old variables
		
		$smarty->assign(array(	'subject'=>'',
					'content'=>'',
					'categories' =>array(),
					'filed_under'=>'',
					'date'=>'',
					'author'=>'',
					'version'=>'',
					'ip-address'=>''
					)
				);
		
		
		if (isset($params['content']) && is_array($params['content']) && $params['content']) {
			//foreach ($params['entry'] as $k => $val)
			$smarty->assign(array_change_key_case($params['content'], CASE_LOWER));
			return $content;
		}
		
		if (isset($params['alwaysshow']) && $params['alwaysshow']) {
			return $content;
		}
		
		if($repeat=$fpdb->hasMore()) {
			$couplet =& $fpdb->getEntry() ;
			
			$id =& $couplet[0];
			$entry =& $couplet[1];
			
			if (THEME_LEGACY_MODE) {
				$entry = theme_entry_filters($entry, $id);
			}
			
			
			foreach($entry as $k=>$v) 
				$smarty->assign_by_ref($k, $entry[$k]);
			
			$smarty->assign_by_ref('id', $id);
		}
		

		return $content;
		
	}
	
	function smarty_block_comment($params, $content, &$smarty, &$repeat) {
		global $fpdb;
		
		// clean old variables
		
		$smarty->assign(array(
					'subject'=>'',
					'content'=>'',
					'date'=>'',
					'name'=>'',
					'url'=>'',
					'email'=>'',
					'version'=>'',
					'ip-address'=>'',
					'loggedin'=>'',
					)
				);
		
		
		if($repeat=$fpdb->hasMoreComments()) {
			
			$couplet =& $fpdb->getComment();
			
			$id =& $couplet[0];
			$comment =& $couplet[1];

			
			foreach($comment as $k=>$v) {
				$smarty->assign_by_ref($k, $comment[$k]);
			}
			
			if (THEME_LEGACY_MODE) {
				$comment = theme_comments_filters($comment, $id);
			}
			
			$smarty->assign('id', $id);
			
			
			
		}
		
		
		return $content;
		

		
	}
	
	function smarty_block_comments($params, $content, &$smarty, &$repeat) {
		global $fpdb;
		
			$show = $fpdb->getEntryComments();
			$smarty->assign('entryid', $fpdb->getCurrentId(0));
			
					
			if ($show) {
				return $content;
			} else {
				$repeat = false;
			}

	}
	
	function smarty_function_nextpage($params) {
		global $fpdb;
			
		list($caption, $link) = $fpdb->getNextLink();
		if ($link) {
			if (isset($params['admin'])) {
				$link .= '&amp;p=' . $_GET['p'];
			}
			return "<div class=\"alignright\"><a href=\"$link\">$caption</a></div>";
		}
	}

	function smarty_function_prevpage($params) {
		global $fpdb;
		list($caption, $link) =  $fpdb->getPrevLink();
		if ($link) {
			if (isset($params['admin'])) {
				$link .= '&amp;p=' . $_GET['p'];
			}

			return "<div class=\"alignleft\"><a href=\"$link\">$caption</a></div>";
		}
	}

	$_FP_SMARTY->register_block('comment','smarty_block_comment');
	$_FP_SMARTY->register_block('comments','smarty_block_comments');
	$_FP_SMARTY->register_block('comment_block','smarty_block_comments');
	
	$_FP_SMARTY->register_block('entries','smarty_block_entries');
	$_FP_SMARTY->register_block('entry_block','smarty_block_entries');
	
	$_FP_SMARTY->register_block('entry','smarty_block_entry');
	
	$_FP_SMARTY->register_function('nextpage','smarty_function_nextpage');
	$_FP_SMARTY->register_function('prevpage','smarty_function_prevpage');
	
	
	
?>
